@@ -193,7 +193,9 @@ def _msvc14_find_vc2017():
             join(root, "Microsoft Visual Studio", "Installer", "vswhere.exe"),
             "-latest",
             "-prerelease",
+            "-requiresAny",
             "-requires", "Microsoft.VisualStudio.Component.VC.Tools.x86.x64",
+            "-requires", "Microsoft.VisualStudio.Workload.WDExpress",
             "-property", "installationPath",
             "-products", "*",
         ]).decode(encoding="mbcs", errors="strict").strip()
